__author__ = 'gotlium'

import traceback
import tempfile
import socket
import atexit
import time
import sys
import os

from django.core.management.base import BaseCommand
from django.utils.encoding import smart_str

try:
    from django.utils.timezone import now
except ImportError:
    from datetime import datetime

    now = datetime.now

from utils import monkey_mix


class BaseCommandMixin(object):
    def __init__(self):
        self._no_monkey.__init__(self)

        self._start_date = now()
        self._start_time = time.time()
        self._elapsed = None
        self._success = True
        self._error_message = None
        self._traceback = None
        self._show_traceback = False
        self._tempdir = tempfile.gettempdir()
        self._script = sys.argv[1]

    def __mmc_get_lock_file(self):
        return os.path.join(self._tempdir, self._script + '.lock')

    def __mmc_unlock(self):
        if self.__mmc_is_run():
            os.unlink(self.__mmc_get_lock_file())

    def __mmc_lock(self):
        open(self.__mmc_get_lock_file(), 'w').close()

    def __mmc_is_run(self):
        return os.path.exists(self.__mmc_get_lock_file())

    def __mmc_check_is_running(self):
        if self.__mmc_is_run():
            sys.stdout.write('Already running\n')
            sys.exit(-1)

    def __mmc_one_copy(self):
        try:
            from models import MMCScript

            if MMCScript.get_one_copy(self._script):
                self.__mmc_check_is_running()
                self.__mmc_lock()
        except Exception, msg:
            print '[MMC]', msg.__unicode__()

    def execute(self, *args, **options):
        self.__mmc_one_copy()
        atexit.register(self._mmc_atexit_callback)
        self._show_traceback = options.get('traceback', False)

        try:
            self._no_monkey.execute(self, *args, **options)
        except Exception, ex:
            self._success = False
            self._error_message = ex.__unicode__()
            self._traceback = traceback.format_exc()

    def __mmc_store_log(self):
        try:
            from mmc.models import MMCLog

            MMCLog.logging(
                start=self._start_date,
                hostname=socket.gethostname(),
                script=self._script,
                elapsed="%0.2f" % (time.time() - self._start_time),
                success=self._success,
                error_message=self._error_message,
                traceback=self._traceback,
                sys_argv=' '.join(map(unicode, sys.argv))
            )
        except Exception, msg:
            print '[MMC] Logging broken with message:', msg.__unicode__()

    def __mmc_send_mail(self):
        if not self._success:
            from mmc.models import MMCEmail

            MMCEmail.send(self._traceback)

    def __mmc_print_log(self):
        if not self._success:
            if self._show_traceback:
                print self._traceback
            else:
                sys.stderr.write(smart_str(
                    'Error: %s\n' % self.style.ERROR(self._error_message)
                ))
            sys.exit(1)

    def _mmc_atexit_callback(self, *args, **kwargs):
        self.__mmc_store_log()
        self.__mmc_send_mail()
        self.__mmc_unlock()
        self.__mmc_print_log()


def inject_management():
    monkey_mix(BaseCommand, BaseCommandMixin)



__author__ = 'gotlium'

from django.core.mail import send_mail
from django.db import models
from django.conf import settings

from mmc.defaults import SUBJECT


class MMCHost(models.Model):
    created = models.DateField(auto_now=True)
    name = models.CharField(max_length=255, unique=True)
    ignore = models.BooleanField(default=False)

    class Meta:
        verbose_name = 'Host'
        verbose_name_plural = 'Hosts'

    def __unicode__(self):
        return self.name


class MMCScript(models.Model):
    created = models.DateField(auto_now=True)
    name = models.CharField(max_length=255, unique=True)
    ignore = models.BooleanField(default=False)
    one_copy = models.BooleanField(default=False)

    @classmethod
    def get_one_copy(cls, name):
        return cls.objects.get(name=name).one_copy

    class Meta:
        verbose_name = 'Script'
        verbose_name_plural = 'Scripts'

    def __unicode__(self):
        return self.name


class MMCLog(models.Model):
    created = models.DateField(auto_now=True)
    start = models.DateTimeField()
    end = models.DateTimeField(auto_now=True)
    elapsed = models.FloatField()
    hostname = models.ForeignKey(MMCHost)
    script = models.ForeignKey(MMCScript)
    success = models.BooleanField(default=False)
    error_message = models.TextField(blank=True, null=True)
    traceback = models.TextField(blank=True, null=True)
    sys_argv = models.CharField(max_length=255, blank=True, null=True)

    class Meta:
        verbose_name = 'Log'
        verbose_name_plural = 'Logs'

    @classmethod
    def logging(cls, **kwargs):
        kwargs['script'] = MMCScript.objects.get_or_create(
            name=kwargs['script'])[0]
        kwargs['hostname'] = MMCHost.objects.get_or_create(
            name=kwargs['hostname'])[0]
        if not kwargs['script'].ignore and not kwargs['hostname'].ignore:
            cls.objects.create(**kwargs)


class MMCEmail(models.Model):
    created = models.DateField(auto_now=True, editable=False)
    email = models.EmailField()
    is_active = models.BooleanField(default=True)

    class Meta:
        verbose_name = 'Email'
        verbose_name_plural = 'Emails'

    @classmethod
    def send(cls, message):
        try:
            emails = list(cls.objects.values_list(
                'email', flat=True).filter(is_active=True))
            if emails:
                send_mail(
                    SUBJECT, message, settings.DEFAULT_FROM_EMAIL, emails)
        except Exception, msg:
            print '[MMC]', msg.__unicode__()



__author__ = 'gotlium'

import inspect


class MonkeyProxy(object):
    def __init__(self, cls):
        monkey_bases = tuple(
            b._no_monkey for b in cls.__bases__ if hasattr(b, '_no_monkey'))
        for monkey_base in monkey_bases:
            for name, value in monkey_base.__dict__.iteritems():
                setattr(self, name, value)


def monkey_mix(cls, mixin, methods=None):
    assert '_no_monkey' not in cls.__dict__, 'Multi monkey mix not supported'
    cls._no_monkey = MonkeyProxy(cls)

    if methods is None:
        isboundmethod = inspect.ismethod
        methods = inspect.getmembers(mixin, isboundmethod)
    else:
        methods = [(m, getattr(mixin, m)) for m in methods]

    for name, method in methods:
        if hasattr(cls, name):
            setattr(cls._no_monkey, name, getattr(cls, name))
        setattr(cls, name, method.im_func)



# -*- encoding: utf-8 -*-

__author__ = 'gotlium'

from django.conf import settings


def get_settings(key, default):
    return getattr(settings, key, default)


SUBJECT = get_settings('MMC_SUBJECT', '[MMC] Errors')
